/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7611_im_im.h
 * Date: 15.09.2016
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Avnet FMC HDMI IN (ADV7611) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/


#ifndef _ADV7611_IM_HEADER_FILE_
#define _ADV7611_IM_HEADER_FILE_

#ifdef __cplusplus
extern "C" {
#endif

#define I2C_ADV7611_IM_MUX_ADDR             0x70 // 0xE0/0xE1 (PCA9546A)
#define I2C_ADV7611_IM_MUX_DATA             0x04 // ADV7611
#define I2C_IO_EXP_IM_MUX_DATA              0x08 // PCA9555 IO Expander

#define I2C_IO_EXP_IM_BASE_ADDR             0x20  // 0x40/0x41


#define I2C_ADV7611_IM_BASE_ADDR            0x98  // IO
#define I2C_ADV7611_IM_CEC_ADDR             0x80
#define I2C_ADV7611_IM_INFOFRAME_ADDR       0x7C
#define I2C_ADV7611_IM_DPLL_ADDR            0x4C
#define I2C_ADV7611_IM_KSV_ADDR             0x64
#define I2C_ADV7611_IM_EDID_ADDR            0x6C
#define I2C_ADV7611_IM_HDMI_ADDR            0x68
#define I2C_ADV7611_IM_CP_ADDR              0x44

#define ADV7611_IM_RSTN                     0x01

#define ADV7611_IM_REG_VS_INF_PB_0_28       0x6f
#define ADV7611_IM_CABLE_DET_A_RAW          0x01


// video signal locked
#define ADV7611_IM_REG_IO_6A                0x6A  // IO
#define ADV7611_IM_DE_REGEN_LCK_RAW         0x01  // IO, Address 0x6A[0]
#define ADV7611_IM_V_LOCKED_RAW             0x02  // IO, Address 0x6A[1]
#define ADV7611_IM_PLL_LOCKED_RAW           0x40  // IO, Address 0x6A[6]

#define ADV7611_IM_REG_HDMI_07              0x07  // HDMI
#define ADV7611_IM_DE_REGEN_FILTER_LOCKED   0x20  // Addr 68 (HDMI), Address 0x07[5]
#define ADV7611_IM_VERT_FILTER_LOCKED       0x80  // Addr 68 (HDMI), Address 0x07[7]

#define ADV7611_IM_REG_HDMI_04              0x04  // HDMI
#define ADV7611_IM_DE_TMDS_PLL_LOCKED       0x01  // Addr 68 (HDMI), Address 0x04[1]

#define ADV7611_IM_LOCKED                   (ADV7611_IM_DE_TMDS_PLL_LOCKED | ADV7611_IM_DE_REGEN_FILTER_LOCKED | ADV7611_IM_VERT_FILTER_LOCKED)


typedef struct adv7611_im_i2c
{
    char            rw;       // 'r', 'w'
    unsigned char   space;    // address space
    unsigned char   reg;      // register address within the adress space
    unsigned char   data;     // data to write
} adv7611_im_i2c_t;


typedef struct adv7611_im_timing
{
    unsigned int  hav;            // number of active pixels in a line
    unsigned int  hfp;            // horizonatal front porch
    unsigned int  hsw;            // horizonatal sync pulse width
    unsigned int  hbp;            // horizonatal back porch
    unsigned int  hsp;            // horizonatal sync pulse polarity, 0 - nagative, 1 - positive

    unsigned int  vav;            // number of lines
    unsigned int  vfp;            // vertical front porch
    unsigned int  vsw;            // vertical sync pulse width
    unsigned int  vbp;            // vertical back porch
    unsigned int  vsp;            // vertical sync pulse polarity, 0 - nagative, 1 - positive

    float         tmds_freq;      // pixel clock

    unsigned int  is_hdmi;        // 0 - DVI, 1 - HDMI
    unsigned int  is_encrypted;
    unsigned int  is_interlaced;  // 0 - progressive, 1 - interlaced
    unsigned int  color_depth;

    unsigned int  locked;         // incoming video signal is locked
} adv7611_im_timing_t;


// Simple hot plug of HDMII
typedef struct {
    unsigned char is_cable_detected;
    unsigned char is_video_valid;
 //   unsigned char is_unplugged;
} adv7611_im_hot_plug_t;


// int adv7611_im_i2c_write(int i2c_fd, unsigned char dev_addr, unsigned char reg, unsigned char data);
// int adv7611_im_i2c_read(int i2c_fd, unsigned char dev_addr, unsigned char reg, unsigned char print_en);

#include "../i2c/i2c_im.h"

#define adv7611_im_i2c_write  i2c_im_write
#define adv7611_im_i2c_read   i2c_im_read

#define adv7611_im_i2c_sel   i2c_im_sel

int adv7611_im_cable_detect(int i2c_fd);
int adv7611_im_init(int i2c_fd, unsigned char edid[256]);
int adv7611_im_get_lock(int i2c_fd);
int adv7611_im_get_timing(int i2c_fd, adv7611_im_timing_t *tm);
void adv7611_im_print_timing(adv7611_im_timing_t *tm);


#ifdef __cplusplus
}
#endif


#endif
